#!/usr/bin/perl

# lowcal.cgi - A lightweight web calendar
# Copyright (C) 2000, 2003 Dave Sherohman (dave@sherohman.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

use strict;

my $VERSION = '1.9.4';

my %config = (
              anonuser          => 'Not logged in',
              badlogin          => 'Invalid login',
              calendar_name     => 'LowCal Calendar',
              daily_links       => 1,
              default_days      => 7,
              default_details   => 'no',
              default_event     => 'blank_event',
              default_items     => 0,
              default_view      => 'month',
              event_cols        => 80,
              event_dir         => 'events',
              event_rows        => 20,
              null_tags         => 1,
              reverse           => 'no',
              template          => 'default',
              template_dir      => 'templates/',
              userfile          => 'users',
             );

=pod

=head1 NAME

lowcal.cgi - A lightweight web calendar

=head1 DESCRIPTION

LowCal is a web calendar which operates from plain text files rather
than requiring that a database engine be installed.  Nonstandard
modules are also avoided as much as possible so as to minimize the
calendar's dependence on outside code.

=head1 COMMANDS

By default, LowCal generates a view of the current month with all
categories of events displayed.  This can be modified by passing the
following parameters, which are handled in the order listed:

=cut

use CGI qw(:standard);
use CGI::Carp qw (fatalsToBrowser);
$CGI::POST_MAX = 10240;
$CGI::DISABLE_UPLOADS = 1;

use Digest::MD5 qw(md5_hex);
use File::Copy;
use Time::Local;

$ENV{SCRIPT_NAME} =~ m#(^.*/)([^/]*)$#;
my $script_path = $1;
my $script_name = $2;

# Make some lists of month and day names for future reference
my @month_abbr = ('', 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
                   'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
my @month_name = ('', 'January', 'February', 'March', 'April', 'May', 'June',
                  'July', 'August', 'September', 'October', 'November',
                  'December');
my @day_abbr = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
my @day_name = ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday',
                'Friday', 'Saturday');

my $calendar_name    = $config{calendar_name};
my $daily_links      = $config{daily_links};
my $days             = $config{default_days};
my $default_event    = $config{default_event};
my $details          = $config{default_details};
my $event_cols       = $config{event_cols};
my $event_dir        = $config{event_dir};
my $event_rows       = $config{event_rows};
my $items            = $config{items};
my $template         = $config{template};
my $template_dir     = $config{template_dir};
my $view             = $config{default_view};
my %default_template = (initialized => 0);
my $reverse          = $config{reverse};

my $userfile         = $config{userfile};
my $anonuser         = $config{anonuser};
my $badlogin         = $config{badlogin};

print "Content-Type: text/html\n";
#print "Content-Type: text/plain\n";

my $user = param('user');
my $admin = 0;
if ($user) {
  do_login();
} else {
  get_user_cookie();
}

my ($mday, $nmon, $year, $wday) = parse_date();
my @categories = parse_categories();
my %tokens = set_base_tokens($mday, $nmon, $year, $wday);

# Backwards compatibility with old listdays/summarydays params
if (param('listdays') =~ /(\d+)/) {
  $view = 'list';
  $days = $1;
  $details = 'no';
}
if (param('summarydays') =~ /(\d+)/) {
  $view = 'list';
  $days = $1;
  $details = 'yes';
}

$view = lc $1 if param('view') =~ /(month|list)/i;
$details = lc param('details') if param('details');
$days = $1 if param('days') =~ /(\d+)/;
$items = $1 if param('items') =~ /(\d+)/;
$reverse = lc param('reverse') if param('reverse');

if (param('event') =~ /([^`' "]+)/) {
  show_event($1);
#} elsif (param('listdays') =~ /(\d+)/) {
#  list_days($1, 'full');
#} elsif (param('summarydays') =~ /(\d+)/) {
#  list_days($1, 'summary');
} elsif (param('admin')) {
  do_admin(param('admin'));
} elsif ($view eq 'list') {
  list_days($days, $items, $details, $reverse);
} else {
  # Default to monthly view if $view is invalid
  show_month();
}

exit 0;


sub show_event {

=pod

=head2 event

When set to an event ID, displays full information on that event.  Event IDs
are of the form yyyymmdd-TAG, where TAG is an arbitrary label used to
distinguish multiple events on the same date.

=cut

  my $event = $1;
  $tokens{EVENTID} = escapeHTML($event);
  $event =~ /-(.*)/;
  my $tag = $1;
  $tokens{TAG} = $tag;
  $tokens{TAGTIME} = set_tagtime($tag);
  my $escaped = param('eventname');
  $escaped =~ s/<[^>]*>//g;
  $tokens{EVENTNAME} = $escaped;

  if ($event =~ /[^-_0-9a-zA-Z]/) {
    print "Status: 403 Forbidden Event ID\n\n";
    parse_template('err_bad_event');
    exit 0;
  }

  print "\n";
  parse_template('evt_header');
  parse_template($event, $event_dir);
  parse_template('evt_footer');
}

=pod

=head2 listdays, summarydays

If either of these is set, a list of all events will be produced, starting
on the selected date (see below) and continuing for a number of days equal
to the value passed for listdays/summarydays.  These two options are
distinct in that listdays provides full information on each event, while
summarydays produces a list of only the event names.

=cut

sub list_days {
  my $days = $_[0];
  my $items = $_[1] || -1;
  my $details = is_true($_[2]);
  my $reverse = is_true($_[3]);

  my $sec_in_day = 86400; # = 60 * 60 * 24
  my $epochtime = timelocal(0, 0, 12, $mday, $nmon - 1, $year);
  my $epoch_end = $epochtime + ($sec_in_day * ($days - 1));
  my $multiple_days = $days > 1;

  # Get list of files in event directory
  opendir(EVENTS, $event_dir) || die "Can't open directory $event_dir: $!";
  my @events = sort readdir EVENTS;
  closedir EVENTS;
  my $catlist = join ',', @categories;

  # Start date tokens
  my ($startday, $startmon, $startyear, $startwday)
    = (localtime($epochtime))[3..6];
  $startmon += 1; $startyear += 1900;

  $tokens{STARTYEAR} = $startyear;

  $tokens{STARTMONTH} = $startmon;
  $tokens{STARTMONTHABBR} = $month_abbr[$startmon];
  $tokens{STARTMONTHNAME} = $month_name[$startmon];

  $tokens{STARTDAY} = $startday;
  $tokens{STARTDAYABBR} = $day_abbr[$startwday];
  $tokens{STARTDAYNAME} = $day_name[$startwday];

  # End date tokens
  my ($endday, $endmon, $endyear, $endwday) 
    = (localtime($epoch_end))[3..6];
  $endmon += 1; $endyear += 1900;

  $tokens{ENDYEAR} = $endyear;

  $tokens{ENDMONTH} = $endmon;
  $tokens{ENDMONTHABBR} = $month_abbr[$endmon];
  $tokens{ENDMONTHNAME} = $month_name[$endmon];

  $tokens{ENDDAY} = $endday;
  $tokens{ENDDAYABBR} = $day_abbr[$endwday];
  $tokens{ENDDAYNAME} = $day_name[$endwday];

  $tokens{DATERANGE} = $multiple_days 
      ? "$startmon/$startday/$startyear - $endmon/$endday/$endyear"
      : "$startmon/$startday/$startyear";

  print "\n";
  parse_template('list_header');

  if ($reverse) {
    $epochtime = $epoch_end;
    $sec_in_day *= -1;
  }
  while (($days > 0) && $items) {
    my ($mday, $nmon, $year, $wday) = (localtime($epochtime))[3..6];
    $nmon += 1;
    $year += 1900;
    my $datestr = sprintf "%04d%02d%02d", $year, $nmon, $mday;

    my $first = 1;

    my @eventlist = sort grep /^$datestr/, @events;
    @eventlist = reverse @eventlist if $reverse;
    foreach (@eventlist) {
      my $event_id = $_;
      my %evtinfo = scan_event("$event_dir/$event_id");

      if (display_category($evtinfo{category})) {
        if ($first) {
          $tokens{DATE} = "$nmon/$mday/$year";
          $tokens{EDATE} = "$mday-$nmon-$year";

          $tokens{YEAR} = $year;

          $tokens{MONTH} = $nmon;
          $tokens{MONTHABBR} = $month_abbr[$nmon];
          $tokens{MONTHNAME} = $month_name[$nmon];

          $tokens{DAY} = $mday;
          $tokens{DAYABBR} = $day_abbr[$wday];
          $tokens{DAYNAME} = $day_name[$wday];

          parse_template('list_day_sep') if ($multiple_days);

          $first = 0;
        } elsif ($details) {
          parse_template('list_evt_sep');
        }

        if ($evtinfo{location}) {
          print "<a href=\"$evtinfo{location}\">$evtinfo{name}</a><p>";
        } else {
          $event_id =~ /-(.*)/;
          my $tag = $1;
          $tokens{TAG} = $tag;
          $tokens{TAGTIME} = set_tagtime($tag);

          if ($details) {
            parse_template($event_id, $event_dir);
          } else {
            print("<a href=\"$script_name?event=$event_id"
                . "&date=$nmon/$mday/$year&eventname=$evtinfo{escaped}");
            print("&category=$catlist") if $catlist;
            print("\">$evtinfo{name}</a><p>");
          }
        }
      }
      $items--;
      last unless $items;
    }

    $epochtime += $sec_in_day;
    $days--;
  }

  parse_template('list_footer');
}

sub do_admin {

=pod

=head2 admin

If this is set, various administrative functions may be invoked depending
on its value.  These are detailed further below.

=cut

  unless ($admin) {
    print "Status: 403 Permission Denied\n\n";
    parse_template('err_not_admin');
    exit 0;
  }

  my %admin_function = (
                        create => \&admin_create,
                        delete => \&admin_delete,
                        edit => \&admin_edit,
                        list => \&admin_list,
                       );

  my $function = shift;
  if (defined $admin_function{$function}) {
    &{$admin_function{$function}}();
  } else {
    admin_list();
  }
}

sub show_month {

=pod

=head2 monthly view

If none of the above are specified, a monthly view will come up by
default, displaying the month in which the selected date falls.

=cut

  my $cal = `cal $nmon $year`; # Should remove this dependence on cal
  my @weeks = split "\n", $cal;
  shift @weeks; shift @weeks;  # Strip off two header lines from cal's output

  # Get list of files in event directory
  opendir(EVENTS, $event_dir) || die "Can't open directory: $!";
  my @events = sort readdir EVENTS;
  closedir EVENTS;
  my $catlist = join ',', @categories;

  print "\n";
  parse_template('cal_header');
  parse_template('cal_table_header');

  foreach my $week (@weeks) {
    my @days = split ' ', $week;
    parse_template('cal_row_start');

    # Pad @days with leading blanks if it's the first week
    if ($days[0] == 1) {
      foreach (0 .. (6 - $days[-1])) {
        parse_template('day_empty');
      }
    }

    foreach my $day (@days) {
      $tokens{DAY} = $day;
      $tokens{DATE} = "$nmon/$day/$year";
      $tokens{EDATE} = "$day-$nmon-$year";
      parse_template('day_start');
      my $datestr = sprintf "%04d%02d%02d", $year, $nmon, $day;
      my $first = 1;
      foreach (grep /^$datestr/, @events) {
        my $evtid = $_;
        if (open(EVTFILE, "$event_dir/$evtid")) {
          while (<EVTFILE>) {
            if (/<meta .*eventname=\"([^\"]*)\"/i) {
              my $evtname = $1;
              my $category = /category=\"([^\"]*)\"/i ? $1 : "";
              my $location = /location=\"([^\"]*)\"/i ? $1 : "";
              if (display_category($category)) {
                if ($first) {
                  if ($daily_links) {
                    print("<a href=\"$script_name?date=$nmon/$day/$year&listdays=1");
                    print("&category=$catlist") if $catlist;
                    print "\">";
                    parse_template('day_number');
                    print '</a>';
                  } else {
                    parse_template('day_number');
                  }
                  $first = 0;
                }
                if ($location) {
                  print "<a href=\"$location\">$evtname</a><p>";
                } else {
                  my $escaped = $evtname;
                  $escaped =~ s/<[^>]*>//g;
                  print("<a href=\"$script_name?event=$evtid&date=$nmon/$day/$year"
                      . "&eventname=$escaped");
                  print("&category=$catlist") if $catlist;
                  print("\">$evtname</a><p>");  
                }
              }
              last;
            }
          }
          close EVTFILE;
        } else {
          print("$evtid: $!");
        }
      }
      # Print date for days with no events
      parse_template('day_number') if $first;
      parse_template('day_end');
    }

    parse_template('cal_row_end');
  }

  $tokens{DAY} = "$mday";
  $tokens{DATE} = "$nmon/$mday/$year";
  $tokens{EDATE} = "$mday-$nmon-$year";
  parse_template('cal_table_footer');
  parse_template('cal_footer');
}

=pod

=head1 ADMINISTRATIVE FUNCTIONS

=cut

sub admin_create {

=pod

=head2 create

=cut

  my $tag = get_tag();
  my $filename = sprintf "$event_dir/%04d%02d%02d-$tag", $year, $nmon, $mday;
  if (-e $filename) {
    print "Status: 403 File Exists\n\n";
    parse_template('err_file_exists');
    exit 0;
  }

  my $source = "$event_dir/" . param('source');
  unless (-r $source) {
    print "Status: 403 Source Not Readable\n\n";
    parse_template('err_no_source');
    exit 0;
  }

  if (lc param('mode') eq 'link') {
    `ln -s $source $filename`;
  } else {
    umask(0002);
    copy($source, $filename);
  }

  unless (-e $filename) {
    print "\n";
    parse_template('err_create_failed');
    exit 0;
  }
  admin_edit();
}

sub admin_delete {

=pod

=head2 delete

=cut

  my $tag = get_tag();
  my $filename = sprintf "$event_dir/%04d%02d%02d-$tag", $year, $nmon, $mday;
  unless (-w $filename) {
    print "Status: 403 Cannot Delete File\n\n";
    parse_template('err_not_deletable');
    exit 0;
  }

  unlink $filename || print STDERR "$!\n";
  if (-e $filename) {
    print "\n";
    parse_template('err_delete_failed');
    exit 0;
  }
  admin_list();
}

sub admin_edit {

=pod

=head2 edit

=cut

  my $tag = get_tag();
  my $filename = sprintf "$event_dir/%04d%02d%02d-$tag", $year, $nmon, $mday;
  unless ((-r $filename) && (-w $filename)) {
    print "Status: 403 Cannot Edit File\n\n";
    parse_template('err_not_editable');
    exit 0;
  }

  my $evtsource = param('evtsource');
  if ($evtsource) {
    open(EVTFILE, ">$filename") || die "Can't write $filename: $!";
    print EVTFILE $evtsource;
    close EVTFILE;
  } else {
    open(EVTFILE, $filename) || die "Can't open $filename: $!";
    my @event_data = <EVTFILE>;
    close EVTFILE;
    $evtsource = join '', @event_data;
  }

  $tokens{EVENTFILE} = sprintf "%04d%02d%02d-$tag", $year, $nmon, $mday;
  $tokens{EVENTBODY} = $evtsource;
  $tokens{EVENTROWS} = $event_rows;
  $tokens{EVENTCOLS} = $event_cols;
  $tokens{TAG} = $tag;

  print "\n";
  parse_template('admin_edit_header');
  parse_template('admin_edit');
  parse_template('admin_edit_footer');
}

sub admin_list {

=pod

=head2 list

'list' is the default administrative action if an invalid value is
provided.

=cut

  my $basename = sprintf "%04d%02d%02d", $year, $nmon, $mday;

  # Get list of files in event directory
  opendir(EVENTS, $event_dir) || die "Can't open directory: $!";
  my @events = sort readdir EVENTS;
  closedir EVENTS;
  $tokens{DEFAULTEVENT} = $default_event;

  print "\n";
  parse_template('admin_list_header');
  parse_template('admin_table_header');

  my ($evtfile, $tag, $title, $unique, $location, $category);
  foreach (grep /^$basename/, @events) {
    $evtfile = $_;
    /^$basename-(.*)/;
    $tokens{EVENTFILE} = $evtfile;
    $tag = $1 || '(Null)';
    $tokens{TAG} = $tag;
    $tokens{TAGTIME} = set_tagtime($tag);

    my %evtinfo = scan_event("$event_dir/$evtfile");
    $tokens{EVENTNAME} = $evtinfo{name};
    $tokens{ESCAPEDNAME} = $evtinfo{escaped};
    $tokens{CATLIST} = $evtinfo{category};
    $tokens{EVENTLOCATION} = $evtinfo{location} || '(Local)';
    $tokens{UNIQUE} = (-l $evtfile) ? 'L' : 'U';
    parse_template('admin_row');
  }
  parse_template('admin_table_footer');
  parse_template('admin_list_footer');
}


=pod

=head1 OPTIONS

The following options can be used to modify the behaviour of all commands
(including the monthly view which results from not providing an explicit
command):

=cut

=pod

=head2 date, month, day, year

Specify the date to display.  In monthly view, the month in which this date
falls will be displayed.  For listdays/summarydays views, this is the
starting date to list events from.

month, day, and year may be relative.  e.g., "month = -1" will display
the month prior to the given date.  If date is not explicitly specified,
it defaults to the current date.

=cut

sub parse_date {
  my ($day, $month, $year);
  my ($weekday, $day_off, $month_off, $year_off);

  ($day, $month, $year) = (localtime)[3..5];
  $year += 1900;
  $month += 1;

  my ($date_param, $day_param, $month_param, $year_param)
   = (param('date') || '', param('day') || '', param('month') || '',
      param('year') || '');

  # Compensate for URI decoding converting + to space
  $date_param   =~ tr/ /+/;
  $day_param    =~ tr/ /+/;
  $month_param  =~ tr/ /+/;
  $year_param   =~ tr/ /+/;

  if ($date_param) {
    if ($date_param =~ m~(\d\d?)/(\d\d?)/(\d+)~) {
      # American mm/dd/yyyy
      $month = $1;
      $day = $2;
      $year = $3;
    } elsif ($date_param =~ /(\d\d?)-(\d\d?)-(\d+)/) {
      # European dd-mm-yyyy
      $day = $1;
      $month = $2;
      $year = $3;
    }
  }

  # Untaint and handle offsets
  if ($year_param =~ /(\d*)([-+]?)(\d*)/) {
    $year = $1 if $1;
    $year += $3 if $2 eq '+';
    $year -= $3 if $2 eq '-';
  }

  if ($month_param =~ /(\d*)([-+]?)(\d*)/) {
    $month = $1 if $1;
    if ($2 eq '+') {
      $month += $3;
      while ($month > 12) {
        $month -= 12;
        $year += 1;
      }
    } elsif ($2 eq '-') {
      $month -= $3;
      while ($month < 1) {
        $month += 12;
        $year -= 1;
      }
    }

    # Force $day into the range of dates present in $month
    $day = 31 if $day > 31;   # timelocal dies on day-of-month > 31
    my $calc_day =
          (localtime(timelocal(0, 0, 12, $day, ($month - 1), $year)))[3];
          $day -= $calc_day if ($calc_day < $day);  # Wrapped past end of month
        
  }

  if ($day_param =~ /(\d*)([-+]?)(\d*)/) {
    $day = $1 if $1;
    if ($2 eq '+') {
      $day_off = $3;
    } elsif ($2 eq '-') {
      $day_off = $3 * -1;
    }
  }

  # Convert date to epoch time and back to handle day offset and obtain
  # day of week
  my $epochtime = timelocal(0, 0, 12, $day, ($month - 1), $year);
  $epochtime += 24 * 60 * 60 * $day_off;
  ($day, $month, $year, $weekday) = (localtime($epochtime))[3..6];
  $year += 1900;
  $month += 1;

  return ($day, $month, $year, $weekday);
}

=pod

=head2 category, categories

Specifies a list of event categories to display.  Multiple categories can
be specified as either a comma-separated list within a single category
parameter or by providing multiple category parameters.

=cut

sub parse_categories {
  my @catarr = ();
  push @catarr, param('category') if param('category');
  push @catarr, param('categories') if param('categories');
  my $catlist = join ',', @catarr;

  $catlist = '' if $catlist =~ /\*/;  # Empty string and * are equivalent

  return split ',', $catlist;
}

=pod

=head1 TOKENS

Various tokens can be used in templates and event files.  The following tokens
are always available.  Others only apply in certain templates and are listed
under TEMPLATES below.

DATE            Date in U.S. format (month/day/year)
EDATE           Date in Eurpoean format (day-month-year)
YEAR            Year
MONTH           Numeric month
MONTHABBR       Three-letter abbreviation for the month's name
MONTHNAME       Full name of the month
DAY             Doy of month
DAYABBR         Three-letter abbreviation for the weekday
DAYNAME         Full name of weekday
LASTMONTH       Number of the previous month
LASTMONTHYEAR   Year in which the previous month fell
LATMONTHABBR, LASTMONTHNAME
                As MONTHABBR and MONTHNAME for previous month
NEXTMONTH, NEXTMONTHYEAR, NEXTMONTHABBR, NEXTMONTHNAME
                As LASTMONTH, etc., but for following month
CATLIST         List of categories displayed or "all events", formatted
                for human consumption
CATPARAM        List of categories displayed, formatted for use in URIs
DUMPTOKENS      Prints a list of all available tokens (hardcoded formatting;
                not recommended for anything other than testing)
USER            Username currently logged in
ANONSTART, ANONEND
                If a user is logged in, these will be set to HTML comment
                start and end tags.  If no user is logged in, they will be
                empty.  The effect is that anything between these tokens
                will only appear to the user if he is anonymous.
USERSTART, USEREND
                These tokens are the inverse of ANONSTART and ANONEND;
                anything between them will only be displayed if a user is
                logged in.

=cut

sub set_base_tokens {
  my ($day, $month, $year, $weekday) = @_;
  my %tokens = ();

  $tokens{VERSION} = $VERSION;
  $tokens{URI} = $script_name;
  $tokens{CALNAME} = $calendar_name;

  $tokens{DATE} = "$month/$day/$year";
  $tokens{EDATE} = "$day-$month-$year";

  $tokens{YEAR} = $year;
  $tokens{MONTH} = $month;
  $tokens{MONTHABBR} = $month_abbr[$month];
  $tokens{MONTHNAME} = $month_name[$month];

  $tokens{DAY} = $day;
  $tokens{DAYABBR} = $day_abbr[$weekday];
  $tokens{DAYNAME} = $day_name[$weekday];

  my ($omon, $oyear);

  if ($month == 1)      {$omon = 12;            $oyear = $year - 1;}
  else                  {$omon = $month - 1;    $oyear = $year;}
  $tokens{LASTMONTH} = $omon;
  $tokens{LASTMONTHYEAR} = $oyear;
  $tokens{LASTMONTHABBR} = $month_abbr[$omon];
  $tokens{LASTMONTHNAME} = $month_name[$omon];

  if ($month == 12)     {$omon = 1;             $oyear = $year + 1;}
  else                  {$omon = $month + 1;    $oyear = $year;}
  $tokens{NEXTMONTH} = $omon;
  $tokens{NEXTMONTHYEAR} = $oyear;
  $tokens{NEXTMONTHABBR} = $month_abbr[$omon];
  $tokens{NEXTMONTHNAME} = $month_name[$omon];

  $tokens{CATLIST} = ($#categories > -1) ? join ', ', @categories
                                         : 'all events';
  $tokens{CATPARAM} = join ',', @categories;

  $tokens{USER} = $user || $anonuser;
  
  if (($user eq $anonuser) || ($user eq $badlogin)) {
    $tokens{ANONSTART} = '';
    $tokens{ANONEND}   = '';
    $tokens{USERSTART} = '<!--';
    $tokens{USEREND}   = '-->';
  } else {
    $tokens{ANONSTART} = '<!--';
    $tokens{ANONEND}   = '-->';
    $tokens{USERSTART} = '';
    $tokens{USEREND}   = '';
  }

  return %tokens;
}


### Begin podless internal utility functions ###

sub get_tag {
  my $tag = param('tag');
  unless ($tag || $config{null_tags}) {
    print "Status: 406 Tag Required\n\n";
    parse_template('err_no_tag');
    exit 0;
  }

  $tag = '' if $tag eq '(Null)';
  if (lc $tag eq 'now') {
    my ($second, $minute, $hour) = (localtime)[0..2];
    $tag = $hour . $minute . $second;
  }

  if ($tag =~ /[^-_0-9a-zA-Z]/) {
    print "Status: 403 Forbidden Event ID\n\n";
    parse_template('err_bad_event');
    exit 0;
  }

  return $tag;
}

sub set_tagtime {
  my $tag = shift;
  if ($tag =~ /([0-2]\d)([0-5]\d)([0-5]\d)/) {
    $tag = "$1:$2:$3" if $1 < 24;
  }
  return $tag;
}

sub display_category {
  my $category = shift;

  # If category list is empty, display everything
  return 1 unless $#categories > -1;

  foreach my $cat (split ',', $category) {
    return 1 if (grep /$cat/i, @categories);
  }

  return 0;
}

sub do_login {
  my ($usercookie, $passcookie);
  if (lc $user eq 'logout') {
    $usercookie = cookie(-name => 'user', -value => '',
                         -path => $script_path, -expires => '-1y');
    $passcookie = cookie(-name => 'passhash', -value => '',
                         -path => $script_path, -expires => '-1y');
    print "Set-Cookie: $usercookie\n";
    print "Set-Cookie: $passcookie\n";
    $user = $anonuser;
    return;
  }

  my %userdata = get_user_data($user);
  if (md5_hex(param('password') . "\n") eq $userdata{passhash}) {
    do_user_prefs(%userdata);
    if (param('remember')) {
      $usercookie = cookie(-name => 'user', -value => $user,
                           -path => $script_path, -expires => '+1y');
      $passcookie = cookie(-name => 'passhash', -value => $userdata{passhash},
                           -path => $script_path, -expires => '+1y');
    } else {
      $usercookie = cookie(-name => 'user', -value => $user,
                           -path => $script_path);
      $passcookie = cookie(-name => 'passhash', -value => $userdata{passhash},
                           -path => $script_path);
    }
    print "Set-Cookie: $usercookie\n";
    print "Set-Cookie: $passcookie\n";
  } else {
    $usercookie = cookie(-name => 'user', -value => '',
                         -path => $script_path, -expires => '-1y');
    $passcookie = cookie(-name => 'passhash', -value => '',
                         -path => $script_path, -expires => '-1y');
    $user = $badlogin;
  }
}

sub do_user_prefs {
  my %prefs = @_;

  $admin = $prefs{admin} || 0;
  $daily_links = $prefs{dailylinks} if exists $prefs{dailylinks};
  $days = $prefs{defaultdays} if exists $prefs{defaultdays};
  $default_event = $prefs{defaultevent} if exists $prefs{defaultevent};
  $details = $prefs{defaultdetails} if exists $prefs{defaultdetails};
  $event_cols = $prefs{eventcols} if exists $prefs{eventcols};
  $event_rows = $prefs{eventrows} if exists $prefs{eventrows};
  $items = $prefs{items} if exists $prefs{items};
  $reverse = $prefs{reverse} if exists $prefs{reverse};
  $template = $prefs{template} if exists $prefs{template};
  $view = $prefs{defaultview} if exists $prefs{defaultview};

}

sub get_user_cookie {
  if ($user = cookie('user')) {
    my %userdata = get_user_data($user);
    if (cookie('passhash') eq $userdata{passhash}) {
      do_user_prefs(%userdata);
    } else {
      $user = $badlogin;
    }
  } else {
    $user = $anonuser;
  }
}

sub get_user_data {
  my $uname = shift;
  my %userdata = ();

  open (USERS, $userfile) || return [];
  my $line;
  while ($line = <USERS>) {
    chomp $line;
    last if $line =~ /^User:\s*$uname$/;
  }
  unless ($line) {
    close USERS;
    return %userdata;
  }

  $userdata{user} = $uname;
  while (my $line = <USERS>) {
    next if $line =~ /^#/;
    chomp $line;
    last if $line =~ /^\s*$/;
    next unless $line =~ /^([^:]+):\s*(\S.*)$/;
    $userdata{lc $1} = $2;
  }
  close USERS;
  return %userdata;
}

sub scan_event {
  my $evtid = shift;
  my %evtinfo = ();

  if (open(EVTFILE, $evtid)) {
    while (<EVTFILE>) {
      if (/<meta .*eventname=\"([^\"]*)\"/i) {
        $evtinfo{name}     = $1;
        #$evtinfo{escaped}  = escapeHTML($1);
        my $escaped = $1;
        $escaped =~ s/<[^>]*>//g;
        $evtinfo{escaped}  = $escaped;
        $evtinfo{category} = /category=\"([^\"]*)\"/i ? $1 : "";
        $evtinfo{location} = /location=\"([^\"]*)\"/i ? $1 : "";
        last;
      }
    }
    close EVTFILE;
  } else {
    print("$evtid: $!");
  }

  return %evtinfo;
}

sub is_true {
  my $value = lc shift;

  return ($value eq 'yes') || ($value eq 'true') || ($value eq 'y')
      || ($value eq 'on') || ($value + 1 != 1);
}

sub parse_template {
  my $template_name = shift;
  my $dir = shift || ($template_dir . $template);
  $tokens{TEMPLATE} = escapeHTML($template_name);
  my $data = ();

  if (open(TEMPLATE, "$dir/$template_name")) {
    my @data = <TEMPLATE>;
    close TEMPLATE;
    $data = join '', @data
  } else {
    init_default_templates() unless $default_template{'initialized'};
    if (exists $default_template{$template_name}) {
      $data = $default_template{$template_name};
    } else {
      $data = $default_template{'default'};
    }
  }

  if ($data =~ /%\S+%/) {
    while (my ($token, $value) = each %tokens) {
      $data =~ s/%$token%/$value/g;
    }
    if ($data =~ /%DUMPTOKENS%/) {
      my $tokstr = '';
      foreach my $tok (sort keys %tokens) {
        $tokstr .= "$tok: $tokens{$tok}<br>\n";
      }
      $data =~ s/%DUMPTOKENS%/$tokstr/g;
    }
  }

  print $data;
}

sub init_default_templates {
  %default_template = (
    initialized => 1,

    default => "Error: Event or template file %TEMPLATE% has disappeared!\n",

    admin_edit =>
     "<form action=%URI% method=post>\n<table border=1 width=100%>
      <tr valign=top>\n<td>
      <textarea name=evtsource rows=%EVENTROWS% cols=%EVENTCOLS%>
      %EVENTBODY%
      </textarea>\n</td>\n<td>
      %EVENTBODY%
      </td>\n</table>
      <table width=100%><tr>
      <td align=left>
      <input type=hidden name=admin value=edit>
      <input type=hidden name=date value=%DATE%>
      <input type=hidden name=tag value=%TAG%>
      <input type=submit name=submit value=\"Save Changes\">\n</form>
      </td><td align=right>
      <a href=\"%URI%?admin=list&date=%DATE%\">Show events on %DATE%</a>
      </td>\n",

    admin_edit_footer => "</body></html>",

    admin_edit_header =>
     "<html>\n<head>\n<title>Event Data: %EVENTFILE%</title>\n<body>\n",

    admin_list_footer => "</body></html>",

    admin_list_header => 
     "<html><head>\n<title>Event List</title>\n</head><body>
      <h3 align=center>Events for %DAYNAME%, %MONTHNAME% %DAY%, %YEAR%</h3>\n",

    admin_row =>
     "<tr>\n<td>%TAG%</td>\n<td>
      <a href=\"%URI%?event=%EVENTFILE%&date=%DATE%&eventname=%EVENTNAME%\">
      %EVENTNAME%</a></td>
      <td align=center>%UNIQUE%</td>
      <td>%EVENTLOCATION%</td><td>%CATLIST%</td><td>
      <a href=\"%URI%?admin=edit&tag=%TAG%&date=%DATE%\">Edit</a>
      </td><td>
      <a href=\"%URI%?admin=delete&tag=%TAG%&date=%DATE%\">Delete</a>
      </td></tr>\n\n",

    admin_table_footer =>
     "</table>\n<p>\n<table border=0 width=100%>\n<tr>
      <td align=left>\n<form action=%URI% method=post>\n<table>\n
      <tr><td colspan=2><h3 align=center>Create new event</h3></td></tr>
      <tr><td>Event tag:</td><td><input name=tag></td></tr>
      <tr><td>Source file:</td>
          <td><input name=source value=%DEFAULTEVENT%></td></tr>
      <input name=date type=hidden value=%DATE%>
      <input name=admin type=hidden value=create>
      </table>\n<table>\n<tr>
      <td width=50%>
      <input name=mode value=Copy type=submit> from source file</td>
      <td width=50%>
      <input name=mode value=Link type=submit> to source file</td>
      </tr>\n</table>\n</form>\n</td>\n<td align=right valign=top>\n
      <form action=%URI% method=get>
      <input type=hidden name=admin value=list>
      <input name=date value=%DATE%>
      <input type=submit name=setdate value=\"Show Date\">
      </form>\n</td>\n</tr>\n</table>\n\n",

    admin_table_header =>
     "<table border=1>\n<th>Event Tag</th>\n<th>Event Name</th>
      <th>U/L</th>\n<th>Location</th>\n<th>Categories</th>\n",

    cal_footer => "</body>\n</html>\n",

    cal_header =>
     "<html>\n<head>\n<title>%CALNAME%</title>\n</head>\n<body>\n",

    cal_row_end => "</tr>\n",

    cal_row_start => "<tr valign=top>\n",

    cal_table_footer => "</table>\n\n",

    cal_table_header =>
     "<table border=1 align=center width=95%>\n<tr valign=top>
      <td align=center><b>Sunday</b></td>
      <td align=center><b>Monday</b></td>
      <td align=center><b>Tuesday</b></td>
      <td align=center><b>Wednesday</b></td>
      <td align=center><b>Thursday</b></td>
      <td align=center><b>Friday</b></td>
      <td align=center><b>Saturday</b></td>
      </tr>\n",

    day_empty => "<td>&nbsp;</td>",

    day_end => "</td>\n",

    day_number => "%DAY%</p>\n",

    day_start => "<td width=14%><p align=right>",

    err_bad_event =>
     "<html>\n<head>\n<title>403 Forbidden Event ID</title>\n</head>
      <body>\n<h1>Forbidden Event ID</h1>\n<p>
      Event IDs and tags may only contain letters, numbers, hyphens,
      and underscores.
      </p>\n</body>\n</html>\n",

    err_create_failed =>
     "<html>\n<head>\n<title>File Creation Failed</title>\n</head>
      <body>\n<h1>Forbidden Event ID</h1>\n<p>
      The web server was unable to create a new event file.
      </p>\n</body>\n</html>\n",

    err_delete_failed =>
     "<html>\n<head>\n<title>Deletion Failed</title>\n</head>
      <body>\n<h1>Deletion Failed</h1>\n<p>
      The web server was unable to delete the requested file.
      </p>\n</body>\n</html>\n",

    err_file_exists =>
     "<html>\n<head>\n<title>403 File Exists</title>\n</head>
      <body>\n<h1>File Exists</h1>\n<p>
      An event file already exists on the specified date with the
      requested tag.
      </p>\n</body>\n</html>\n",

    err_no_source =>
     "<html>\n<head>\n<title>403 Source Not Readable</title>\n</head>
      <body>\n<h1>Source Not Readable</h1>\n<p>
      The specified source file does not exist or is not readable by the
      web server.
      </p>\n</body>\n</html>\n",

    err_no_tag =>
     "<html>\n<head>\n<title>406 Tag Required</title>\n</head>
      <body>\n<h1>Tag Required</h1>\n<p>
      The <i>tag</i> parameter must be set on this request.
      </p>\n</body>\n</html>\n",

    err_not_admin =>
     "<html>\n<head>\n<title>403 Permission Denied</title>\n</head>
      <body>\n<h1>Permission Denied</h1>\n<p>
      This function is only accessible to users with administrative
      privileges.  Please log in as such a user and try again.
      </p>\n</body>\n</html>\n",

    err_not_deletable =>
     "<html>\n<head>\n<title>403 Cannot Delete File</title>\n</head>
      <body>\n<h1>Cannot Delete File</h1>\n<p>
      Deleting an event file requires that the file must exist and be
      writable by the web server.
      </p>\n</body>\n</html>\n",

    err_not_editable =>
     "<html>\n<head>\n<title>403 Cannot Edit File</title>\n</head>
      <body>\n<h1>Cannot Edit File</h1>\n<p>
      Editing an event file requires that the file must exist and be
      writable by the web server.
      </p>\n</body>\n</html>\n",

    evt_footer => "</body>\n</html>\n",

    evt_header =>
     "<html>\n<head>\n<title>%CALNAME%</title>\n</head>\n<body>\n",

    list_day_sep => "<hr>\n<h3>%DATE% -</h3>\n",

    list_evt_sep => "<hr width=50%>\n",

    list_footer => "<hr>\n</body>\n</html>\n",

    list_header =>
     "<html>\n<head>\n<title>%CALNAME% Event List</title>\n</head>
      <body>\n",

    show_all_tokens => "%DUMPTOKENS%\n",

  );
  foreach my $key (keys %default_template) {
    $default_template{$key} =~ s/      //g;
  }
}

